from .static_files.FKT_LIB_CureKin import *
from .static_files.CureKin_Material_Models import *
from .static_files.ST_FiberVR_STApp import *
from .static_files.ST_PyCOMP_STApp import *
from .static_files.ST_HoGeneS_STApp import *
from .static_files.FKT_LIB_GFEM import *
import shutil
import math
from numpy.linalg import norm as vecnorm
import numpy as np
import os

def SimManufac(fvc: float, time_data: list, temperature_data: list, defect_list: list, working_directory: str) -> list:
    """Perform process simulation and create CCX input files.

    Args:
        fvc: percent fibre volume content 0%-100%
        time_data: list of float time values for temperature timeline (hours)
        temperature_data: list of float temperature values for temperature timeline (°C)
        defect_list: list of defect dictionaries
        working_directory: relative path to write output files

    Returns:
        list of relative file paths of relevant output files
    """
    fvc = float(fvc)/100
    shutil.copytree(os.path.dirname(os.path.realpath(__file__)) + "/dynamic_files", working_directory + "/dynamic_files")
    
    dir_dyn = working_directory + '/dynamic_files/'
    dir_sta = os.path.dirname(os.path.realpath(__file__)) + '/static_files/'

    out_paths = []
    
    resSIG_dico = {}
    CMat_dico = {}

    sec_temp_lst = []
    for time in time_data:
        sec_temp_lst.append(float(int(float(time) *  3600.)))
    time_data = sec_temp_lst
    
    linewise = read_file(str(dir_sta + '/ShellSection_ElementID_2024-08-21_lug01.incl'))
    Sec_dico = ShSec_transl(linewise)

    linewise = read_file(str(dir_sta + 'Orientation_ElementID.incl'))
    Ori_dico = ORI_transl(linewise)

    MRUC_geom_dico = {}  
    
    def_wavin, def_misori, def_plymiss, def_thickn = prep_defect_list(defect_list)

    linewise = read_file(str(dir_sta + '/MRUC_MAT_DEF_2024-08-01_lug01.inpy'))
    MRUC_lst = []
    f_lenZ = 1.0
    f_lenZ_lst = []
    for line in linewise:
        if line[0][0:2] != '**':
            separ = line[0].split(',')
            separ.append(f_lenZ)
            MRUC_lst.append(separ)
            f_lenZ_lst.append(f_lenZ)
 
    mruc_f_lenZ = []
    for MRUC in MRUC_lst:
        msic = str(MRUC[0].replace(' ','') )
        fvc_tow, V_tow, V_resin, lenX, lenY, lenZ =  ST_FiberVR(dir_sta, msic, 'PURERESIN', fvc, f_lenZ )
        MRUC_geom_dico.update({msic.split('.')[0]:[fvc_tow, V_tow, V_resin, lenX, lenY, lenZ, f_lenZ   ]})

    for defect in def_thickn:
        elm = defect[0]
        lay = defect[1]
        fvd = defect[2]

        matd = Sec_dico[str(elm)][lay][2]

        Dh = (fvc / fvd - 1)
        f_lenZ = Dh + 1
        
        newlines = []
        lin_count = 0
        for line in Sec_dico[str(elm)]:
            if lin_count != lay:
                newlines.append(line)
            else:
                newlines.append([str(float(line[0]) * f_lenZ), line[1], str(matd + '__fvd' + str(fvd * 100).replace('.','p')), line[3] ]  )
            lin_count += 1
        Sec_dico.update({str(elm):newlines})
        
        for ruc in MRUC_lst:
            if ruc[0] == matd:
                new_ruc = [str(matd + '__fvd' + str(fvd * 100).replace('.','p')), ruc[1], ruc[2], ruc[3], f_lenZ ]
                MRUC_lst.append(new_ruc)
        f_lenZ_lst.append(f_lenZ)
   
    mruc_count = 0
    for MRUC in MRUC_lst:
        mruc_count += 1
        f_lenZ = MRUC[4]

        msic = str(MRUC[0].replace(' ','') ).split('__')[0]
        
        fvc_tow, V_tow, V_resin, lenX, lenY, lenZ =  ST_FiberVR(dir_sta, msic, 'PURERESIN', fvc, f_lenZ )

        MRUC_geom_dico.update({msic.split('.')[0]:[fvc_tow, V_tow, V_resin, lenX, lenY, lenZ , f_lenZ   ]})
        
        mat_param_dico = {'V_f': fvc_tow}
        param_lst = ['V_f']

        resin_respar = str(MRUC[1].replace(' ',''))
        fiber_respar = str(MRUC[2].replace(' ',''))
        
        linewise = read_file(str(dir_sta + resin_respar) )       
        mat_dico, param_lst_temp = respar_transl(linewise)
        
        mat_param_dico.update(mat_dico)
        
        for i in param_lst_temp:
            param_lst.append(i)
        
        linewise = read_file(str(dir_sta + fiber_respar) )        
        fiber_dico, fiber_lst_temp = respar_transl(linewise)
        
        mat_param_dico.update(fiber_dico)
        
        for i in fiber_lst_temp:
            param_lst.append(i)

        fin_dico = ST_PyCOMP_STApp(dir_dyn, dir_sta, time_data, temperature_data, mat_param_dico, param_lst, msic)

        effP_tiles_lst, X_lst, Y_lst, Z_lst, msic_lst = ST_HoGeneS_STApp(dir_dyn, dir_sta, mat_param_dico, param_lst, fin_dico, msic + '.msic', f_lenZ )

        Dx_lst = []
        Dy_lst = []
        Dz_lst = []
    
        for i in range(1, len(X_lst)):        
            Dx_lst.append( ( X_lst[i] - X_lst[i-1] ) / X_lst[-1] )
            
        for i in range(1, len(Y_lst)):        
            Dy_lst.append( ( Y_lst[i] - Y_lst[i-1] ) / Y_lst[-1] )
            
        for i in range(1, len(Z_lst)):        
            Dz_lst.append( ( Z_lst[i] - Z_lst[i-1] ) / Z_lst[-1] )

        E1 = 0.
        E2 = 0.
        E3 = 0.
        NUE12 = 0.
        NUE13 = 0.
        NUE23 = 0.
        G12 = 0.
        G13 = 0.
        G23 = 0.
            
        for ylen in range(0, len(Dy_lst)):
            for xlen in range(0, len(Dx_lst)):

                E1 += effP_tiles_lst[ylen][xlen][0]  *  Dy_lst[ylen] * Dx_lst[xlen]
                E2 += effP_tiles_lst[ylen][xlen][1]  *  Dy_lst[ylen] * Dx_lst[xlen]
                E3 += effP_tiles_lst[ylen][xlen][2]  *  Dy_lst[ylen] * Dx_lst[xlen]
                
                NUE12 += effP_tiles_lst[ylen][xlen][3]  *  Dy_lst[ylen] * Dx_lst[xlen]
                NUE13 += effP_tiles_lst[ylen][xlen][4]  *  Dy_lst[ylen] * Dx_lst[xlen]
                NUE23 += effP_tiles_lst[ylen][xlen][5]  *  Dy_lst[ylen] * Dx_lst[xlen]
                                
                G12 += effP_tiles_lst[ylen][xlen][6]  *  Dy_lst[ylen] * Dx_lst[xlen]
                G13 += effP_tiles_lst[ylen][xlen][7]  *  Dy_lst[ylen] * Dx_lst[xlen]
                G23 += effP_tiles_lst[ylen][xlen][8]  *  Dy_lst[ylen] * Dx_lst[xlen]

        EPS_Yarn_YZ = fin_dico['EPS_Y_norm']
        EPS_Yarn_X = -fin_dico['EPS_Y_par']

        RSIG_ori_lst = []
        SX_tot = 0.
        SY_tot = 0.
        SZ_tot = 0.
        for y_row in range(0, len(Dy_lst)):
            y_row_lst = []
            for tile in range(0, len(Dx_lst)):
                tile_lst = []

                SX = 0.
                SY = 0.
                SZ = 0.
                for lay in range(0, len(Dz_lst)):

                    volume = Dy_lst[y_row] * Dx_lst[tile] * Dz_lst[lay]

                    ori = msic_lst[y_row][tile][lay][1]
                    RelVol = Dy_lst[y_row] * Dx_lst[tile] * Dz_lst[lay]

                    if float(ori) == 0.0 and msic_lst[y_row][tile][lay][0] != 'PURERESIN':
                        SX += -EPS_Yarn_X  * fin_dico['CH_E11'] *RelVol
                        SY += -EPS_Yarn_YZ * fin_dico['CH_E22'] *RelVol
                        SZ += -EPS_Yarn_YZ * fin_dico['CH_E22'] *RelVol
                        
                    elif float(ori) == 90.0 and msic_lst[y_row][tile][lay][0] != 'PURERESIN':
                        SX += -EPS_Yarn_YZ * fin_dico['CH_E22']  *RelVol
                        SY += -EPS_Yarn_X  * fin_dico['CH_E11']  *RelVol
                        SZ += -EPS_Yarn_YZ * fin_dico['CH_E22']  *RelVol
                    else: 
                        SX += -fin_dico['EPS_r_tot'] *RelVol
                        SY += -fin_dico['EPS_r_tot'] *RelVol
                        SZ += -fin_dico['EPS_r_tot'] *RelVol
                        
                    SX_tot += SX
                    SY_tot += SY
                    SZ_tot += SZ
                    
                    tile_lst.append([SX, SY, SZ])
                y_row_lst.append(tile_lst)
            RSIG_ori_lst.append(y_row_lst)

        resSIG_dico.update({MRUC[0]:[SX_tot, SY_tot, SZ_tot]})
        
        MCardH = str('*MATERIAL, NAME=' + MRUC[0])
        matcard_lst = [  
            [str('*MATERIAL, NAME=' + MRUC[0])],
            ['*DENSITY'],
            [str(MRUC[3])],
            ['*ELASTIC, TYPE=ENGINEERING CONSTANTS'],
            [str(str(E1) +', '+ str(E2) +', '+ str(E3) +', '+ str(NUE12) +', '+ str(NUE13) +', '+ str(NUE23) +', '+ str(G12) +', '+ str(G13)) ],
            [str(G23)]
               ]
        CMat_dico.update({MRUC[0]:matcard_lst})
    
    wav_count = 0
    for defect in def_wavin:
        elm = defect[0]
        lay = defect[1]
        ridge_angle = defect[2]
        ampli = defect[3]
        leng = defect[4]

        wave_angle = math.sin(2 * ampli / leng)
        
        matd = Sec_dico[str(elm)][lay][2]
        thic = float(Sec_dico[str(elm)][lay][0])
        dens = CMat_dico[matd][2][0]

        E1 = float(CMat_dico[matd][4][0].split(', ')[0])
        E2 = float(CMat_dico[matd][4][0].split(', ')[1])
        E3 = float(CMat_dico[matd][4][0].split(', ')[2])
        
        nue12 = float(CMat_dico[matd][4][0].split(', ')[3])
        nue13 = float(CMat_dico[matd][4][0].split(', ')[4])
        nue23 = float(CMat_dico[matd][4][0].split(', ')[5])
        
        G12 = float(CMat_dico[matd][4][0].split(', ')[6])
        G13 = float(CMat_dico[matd][4][0].split(', ')[7])
        G23 = float(CMat_dico[matd][5][0].split(', ')[0])
        
        DxZ = wavinator(E1, E2, E3, nue12, nue13,nue23, G12, G13, G23, thic, ridge_angle, wave_angle)
        
        new_matd = str(matd + '__' + str(str(elm) +'_No' + str(wav_count)))
        
        resSIG_dico.update({new_matd : resSIG_dico[matd]})
        
        newlines = []
        lin_count = 0
        for line in Sec_dico[str(elm)]:
            if lin_count != lay:
                newlines.append(line)
            else:
                newlines.append([str((1* ampli/3) + thic), line[1], new_matd ,line[3]    ])
            lin_count += 1
        Sec_dico.update({str(elm):newlines})

        wav_CMat_dic = {
            new_matd : [ 
                [str('*MATERIAL, NAME=' + new_matd)],
                ['*DENSITY'],
                [dens],
                ['*ELASTIC, TYPE=ANISO'],
                [
                    str(str(float(int(DxZ[0, 0]))) + ', ' + 
                    str(float(int(DxZ[0, 1]))) + ', ' + 
                    str(float(int(DxZ[1, 1]))) + ', ' + 
                    str(float(int(DxZ[0, 2]))) + ', ' + 
                    str(float(int(DxZ[1, 2]))) + ', ' + 
                    str(float(int(DxZ[2, 2]))) + ', ' + 
                    str(float(int(DxZ[0, 3]))) + ', ' + 
                    str(float(int(DxZ[1, 3])))) 
                ],
                [
                    str(str(float(int(DxZ[2, 3]))) + ', ' +
                    str(float(int(DxZ[3, 3]))) + ', ' +
                    str(float(int(DxZ[0, 4]))) + ', ' +
                    str(float(int(DxZ[1, 4]))) + ', ' +
                    str(float(int(DxZ[2, 4]))) + ', ' +
                    str(float(int(DxZ[3, 4]))) + ', ' +
                    str(float(int(DxZ[4, 4]))) + ', ' +
                    str(float(int(DxZ[0, 5]))))  
                ],
                [
                    str(str(float(int(DxZ[1, 5]))) + ', ' +
                    str(float(int(DxZ[2, 5]))) + ', ' +
                    str(float(int(DxZ[3, 5]))) + ', ' +
                    str(float(int(DxZ[4, 5]))) + ', ' +
                    str(float(int(DxZ[5, 5]))) + ', 20.0')
                ],
            ]
        }
        
        CMat_dico.update(wav_CMat_dic)
        wav_count += 1
                    
    ot_str = '**\n**\n'        

    for mat in CMat_dico.keys():
        for line in CMat_dico[mat]:
            ot_str += str(line[0] + '\n')
        ot_str += '**\n**\n' 
        
    ot_str += '**'

    ot_str += '\n**\n**\n**\n'
    ot_str += '*MATERIAL, NAME=PURERESIN\n*ELASTIC\n'
    ot_str += str( str(fin_dico['E_r']) + ', ' +  str(fin_dico['nue_r']) + '\n')
    ot_str += '*DENSITY\n1110.0\n**\n**'
    
    out_path = dir_dyn + str('FRP_MATERIAL.incl')
    of = open(out_path, "w")     
    of.write(ot_str)
    of.flush()
    of.close()
    out_paths.append(out_path)


    for defect in def_misori:
        elm = defect[0]
        lay = defect[1]
        angle = defect[2]

        ori = Sec_dico[str(elm)][lay][3]
        CID = Ori_dico[ori]

        x_axis = [CID[0], CID[1], CID[2]]
        y_axis = [CID[3], CID[4], CID[5]]
        z_axis = np.cross(x_axis, y_axis )

        x_axis_rot = VectorRotation(x_axis,z_axis,angle)
        y_axis_rot = VectorRotation(y_axis,z_axis,angle)
        
        Ori_dico.update({str(ori + '__' + str(str(angle).replace('.', 'p'))): [x_axis_rot[0], x_axis_rot[1], x_axis_rot[2], y_axis_rot[0], y_axis_rot[1], y_axis_rot[2]]})
    
        newlines = []
        lin_count = 0
        for line in Sec_dico[str(elm)]:
            if lin_count != lay:
                newlines.append(line)
            else:
                newlines.append([str(float(line[0])), line[1], line[2] , str(ori + '__' + str(str(angle).replace('.', 'p'))) ]  )
            lin_count += 1
        Sec_dico.update({str(elm):newlines})
    
    for defect in def_plymiss:
        elm = defect[0]
        lay = defect[1]

        newlines = []
        lin_count = 0
        for line in Sec_dico[str(elm)]:
            if lin_count != lay:
                newlines.append(line)
            else:
                newlines.append([str(float(line[0])), line[1], 'PURERESIN', line[3] ]  )
            lin_count += 1
        Sec_dico.update({str(elm):newlines})
        
    elm_lst = []
    for elmID in Sec_dico.keys():
        elm_lst.append(int(elmID))
    elm_lst.sort()

    resSIG_dico.update({'PURERESIN' : [-fin_dico['EPS_r_tot'] * 1/5. * fin_dico['E_r'], -fin_dico['EPS_r_tot'] * 1/4. * fin_dico['E_r'], -fin_dico['EPS_r_tot'] * 1/4. * fin_dico['E_r']]})
    
    out_path = dir_dyn + str('RESIDUAL_STRESSES.incl')
    of = open(out_path, "w")
    of.write('*INITIAL CONDITIONS, TYPE=STRESS\n')
    
    ot_str = '**\n'
    mat_lst = CMat_dico.keys()

    for elm in elm_lst:
        
        ot_str = '**\n'
        lay_lst = Sec_dico[str(elm)]
        
        intpnt = 0
        intpnt_lst = []

        lc = 0

        for lay in lay_lst:
            intpnt_lst.append([lc*8 +1, lc*8 +2,lc*8 +3,lc*8 +4,lc*8 +5,lc*8 +6,lc*8 +7,lc*8 +8,])
            lc += 1

        laynum = 0
        for lay in intpnt_lst:
            mat = Sec_dico[str(elm)][laynum][2]
            orinam  = Sec_dico[str(elm)][laynum][3]

            if mat not in resSIG_dico.keys():
                rot_resSIG = [0., 0., 0.]

            if mat in resSIG_dico.keys():
                
                CID = Ori_dico[orinam]
                
                x_axis = [CID[0], CID[1], CID[2]]
                y_axis = [CID[3], CID[4], CID[5]]
                z_axis = np.cross(x_axis, y_axis )
                
                ex = [1., 0., 0.]
                ey = [0., 1., 0.]
                ez = [0., 0., 1.]

                resSIG =  resSIG_dico[mat]

                rot_resSIG = SR_vec_rot(ex, ey, ez, x_axis, y_axis, z_axis, resSIG)

            for intpnt in intpnt_lst[laynum]:
                ot_str += str(str(elm) + ', ' + 
                          str(intpnt)+ ', ' +
                          str(float(abs(int(rot_resSIG[0])))) + ', ' +
                          str(float(abs(int(rot_resSIG[1])))) + ', ' +
                          str(float(abs(int(rot_resSIG[2]))))  + ', 0.0, 0.0, 0.0\n')
                
            laynum += 1

        of.write(ot_str)
         
    of.write('**\n**')
    of.flush()
    of.close()
    out_paths.append(out_path)

 
    out_path = dir_dyn + str('SHELL_SECTIONS.incl')
    of = open(out_path, "w")
    
    elm_lst = []
    for ssec in Sec_dico.keys():
        elm_lst.append(int(ssec))
        elm_lst.sort()
        
    for elm in elm_lst:
        ot_str = ('**\n')
        laysec = Sec_dico[str(elm)]

        ot_str += '*SHELL SECTION, ELSET=Elset_' + str(elm) + ', COMPOSITE, OFFSET=-0.5\n'
        for lay in laysec:
            for i in range(0, len(lay)):
                ot_str += lay[i]
                if i < len(lay)-1 : ot_str += ', '
            ot_str += '\n'
        of.write(ot_str)
    
    of.write('**\n**')
    of.flush()
    of.close()
    out_paths.append(out_path)


    out_path = dir_dyn + str('ORIENTATIONS.incl')
    of = open(out_path, "w")
    
    ori_lst = []
    for ori in Ori_dico.keys():
        ori_lst.append(ori)
        ori_lst.sort()

    for ori in ori_lst:
        ot_str = ('**\n')
        ot_str += '*ORIENTATION, NAME=' + ori + '\n'
        for i in range(0, len(Ori_dico[ori])):
            ot_str += str(Ori_dico[ori][i])
            if i < len(Ori_dico[ori])-1: ot_str += ', '      
        ot_str += '\n'
        of.write(ot_str)
    
    of.write('**\n**')
    of.flush()
    of.close()
    out_paths.append(out_path)

    return [dir_dyn + file for file in os.listdir(dir_dyn)]
    #return out_paths