import math
from cardano_method import CubicEquation

def LTS__T_g(t, T, mpd, X):
    T_g = mpd['T_g0'] + ( (mpd['T_g_inf'] - mpd['T_g0']) * mpd['lambda'] * X   )   /  (  1 - (1 - mpd['lambda']) *  X  )
    return(T_g)


def LTS__m(t, T, mpd, X):    
    m = mpd['m_0'] + mpd['a_mT'] * T + mpd['b_mT'] * (T**2)
    return(m)


def LTS__n(t, T, mpd, X):    
    n = mpd['n_0'] + mpd['a_nT'] * T + mpd['b_nT'] * (T**2)
    return(n)


def LTS__k_c1(t, T, mpd, X):    
    k_c1 = mpd['A_C1'] * math.exp( -(mpd['E_C1']) / (mpd['R'] * (T + 273.15)   ))
    return(k_c1)


def LTS__k_c2(t, T, mpd, X):    
    k_c2 = mpd['A_C2'] * math.exp( -(mpd['E_C2']) / (mpd['R'] * (T + 273.15)   ))
    return(k_c2)


def LTS__f_of_X_T(t, T, mpd, X):
    f_of_X_T = 1 / ( 1 + math.exp( mpd['C'] * (X - (  mpd['a_c0'] + mpd['a_cT'] * (T + 273.15) ))) )
    return(f_of_X_T)


def LTS__l(t, T, mpd, X):
    l = mpd['l_0'] * (T**mpd['p_l'])
    return(l)


def LTS__dX_dt(t, T, mpd, X, hlp):
    dX_dt = ( hlp[5] * (1 - X)**hlp[8]    +    hlp[6] * (X**hlp[3]) * (1-X)**hlp[4]  )
    return(dX_dt)


def LTS__V_r_vol(t, T, mpd, X):
    if X <= mpd['X_0']:
        V_r = mpd['V_r0']
    if X > mpd['X_0']: 
        V_r = ( X - mpd['X_0']   )  * (mpd['V_rmax'] - mpd['V_r0']) / (mpd['X_max'] - mpd['X_0'])
    return(V_r)    


def LTS__EPS_r_vol(t, T, mpd, X, hlp):
    EPS_r_vol = -(((hlp[11] + 1)**(1/3)) - 1)    
    return(EPS_r_vol)


def LTS__EPS_r_vol_incr(t, T, mpd, X, hlp, hlp_inc):
    EPS_r_vol_incr = hlp[12] - hlp_inc[12] 
    return(EPS_r_vol_incr)
    

def LTS__EPS_r_T_incr(t, T, mpd, X, hlp, hlp_inc):
    EPS_r_T_incr = (hlp[1] - hlp_inc[1] )* hlp[14]
    return(EPS_r_T_incr)


def LTS__EPS_r_tot(t, T, mpd, X, hlp, hlp_inc):
    EPS_r_tot =  hlp_inc[16] + hlp[13] + hlp[15]  
    return(EPS_r_tot)


def LTS__E_r(t, T, mpd, X, hlp):

    if hlp[10] <= mpd['X_gel']:
        E_r = mpd['E_r_min']
    if hlp[10] > mpd['X_gel'] and hlp[1] - hlp[2] < mpd['T1']:
        E_r = mpd['E1']
    if hlp[10] > mpd['X_gel'] and mpd['T1'] <= hlp[1] - hlp[2] and hlp[1] - hlp[2] < mpd['T2']:
        E_r = mpd['E2'] + (mpd['E1'] - mpd['E2']) * ( ( hlp[1] - hlp[2] - mpd['T2']  )  /  ( mpd['T1'] - mpd['T2'] )  )
    if hlp[10] > mpd['X_gel'] and mpd['T2'] <= hlp[1] - hlp[2] and hlp[1] - hlp[2] < mpd['T3']:
        E_r = mpd['E3'] + (mpd['E2'] - mpd['E3']) * ( ( hlp[1] - hlp[2] - mpd['T3']  )  /  ( mpd['T2'] - mpd['T3'] )  )
    if hlp[10] > mpd['X_gel'] and mpd['T3'] <= hlp[1] - hlp[2] and hlp[1] - hlp[2] < mpd['T4']:
        E_r = mpd['A'] * math.exp(-(mpd['K']) * (hlp[1] - hlp[2]) )
    if hlp[10] > mpd['X_gel'] and hlp[1] - hlp[2] > mpd['T4']:
        E_r = mpd['E4']

    return(E_r)


def LTS__CTE(t, T, mpd, X, hlp):
    if hlp[10] < mpd['X_gel']:
        CTE = mpd['X_lt_Xgel']
    if hlp[10] >= mpd['X_gel'] and hlp[1] - hlp[2] < 0.:
        CTE = mpd['X_bt_Xgel_and_Tstar_lt_0']
    if hlp[10] >= mpd['X_gel'] and hlp[1] - hlp[2] >= 0.:
        CTE = mpd['X_bt_XgelundTstar_beq_0']

    return(CTE)


def LTS__nue_r(t, T, mpd, X, hlp):
    nue_r = (  mpd['E_r_inf']  - hlp[17] * ( 1 - 2 * mpd['nue_r_inf'])   )  /  ( 2 * mpd['E_r_inf']   )    
    return(nue_r)


def LTS__SIG_0EPS_incr(t, T, mpd, X, hlp, hlp_inc):
    SIG_0EPS_incr =  - ((hlp[17] * (hlp[13] + hlp[15])) / (   1 - 2 * hlp[18] ))
    return(SIG_0EPS_incr)


def LTS__SIG_0EPS(t, T, mpd, X, hlp, hlp_inc):
    SIG_0EPS =  hlp_inc[20] + hlp[19] 
    return(SIG_0EPS)


def SIG_plain_stress_incr(t, T, mpd, X, hlp, hlp_inc):
    SIG_plain_stress_incr =  - ((hlp[17] * (hlp[13] + hlp[15])) / (   1 -  hlp[18] )) 
    return(SIG_plain_stress_incr)


def SIG_plain_stress(t, T, mpd, X, hlp, hlp_inc):
    SIG_plain_stress =  hlp_inc[22] + hlp[21] 
    return(SIG_plain_stress)


def EPS_plain_stress_incr(t, T, mat_param_dico, X, hlp, hlp_inc):
    EPS_plain_stress = (hlp[13] + hlp[15]) * (   ( 2 * hlp[18] )  /  ( 1 - hlp[18] ) + 1    )
    return(EPS_plain_stress)


def EPS_plain_stress(t, T, mat_param_dico, X, hlp, hlp_inc):
    EPS_plain_stress = hlp_inc[24] + hlp[23]
    return(EPS_plain_stress)


def SIG_plain_strain_incr(t, T, mpd, X, hlp, hlp_inc):
    SIG_plain_stress_incr =  - ((hlp[17] * (hlp[13] + hlp[15]))) 
    return(SIG_plain_stress_incr)


def SIG_plain_strain(t, T, mpd, X, hlp, hlp_inc):
    SIG_plain_stress =  hlp_inc[26] + hlp[25] 
    return(SIG_plain_stress)


def EPS_plain_strain_incr(t, T, mat_param_dico, X, hlp, hlp_inc):
    EPS_plain_stress = (hlp[13] + hlp[15]) * ( 1 + hlp[18]   )
    return(EPS_plain_stress)


def EPS_plain_strain(t, T, mat_param_dico, X, hlp, hlp_inc):
    EPS_plain_stress = hlp_inc[28] + hlp[27]
    return(EPS_plain_stress)


def M_StressXY(xflen_lst, yflen_lst, f_m_lst, E_f_trans, E_r, EPS_Y_par, EPS_Y_norm, CTE_f_trans, T, T_f_ref ):
    stressX_lst = []
    stressY_lst = []
    stressZ_lst = []
    stress_IP_lst = []
    
    for i in range(0, len(xflen_lst)):
        if xflen_lst[i] != 0.:
            E_ersX = ( E_f_trans   *  E_r  ) /  ( E_f_trans * (1 - xflen_lst[i])  + (E_r *   xflen_lst[i]))
            dEPSX_T = ( T - T_f_ref) * CTE_f_trans * xflen_lst[i] / (1 - xflen_lst[i])

        if xflen_lst[i] == 0.:
            E_ersX = E_r
            dEPSX_T = 0.

        if yflen_lst[i] != 0.:
            E_ersY = ( E_f_trans   *  E_r  ) /  ( E_f_trans * (1 - yflen_lst[i])  + (E_r *   yflen_lst[i]))
            dEPSY_T = ( T - T_f_ref) * CTE_f_trans * yflen_lst[i] / (1 - yflen_lst[i])
            
        if yflen_lst[i] == 0.:
            E_ersY = E_r
            dEPSY_T = 0.

        stressX_lst.append(-EPS_Y_norm * E_ersY * f_m_lst[i]   - dEPSY_T * E_r * f_m_lst[i] )
        stressY_lst.append(-EPS_Y_norm * E_ersX * f_m_lst[i]  -  dEPSX_T * E_r * f_m_lst[i])
        stressZ_lst.append( EPS_Y_par * f_m_lst[i] * E_r  )
        stress_IP_lst.append(math.sqrt( (-EPS_Y_norm * E_ersX * f_m_lst[i])**2 + (-EPS_Y_norm * E_ersY * f_m_lst[i])**2))
    
    return(stressX_lst, stressY_lst, stressZ_lst, stress_IP_lst)

           
def M_PrincStress(stressX_lst, stressY_lst, stressZ_lst):
    max_princ_stress_lst = []
    min_princ_stress_lst = []
    
    for i in range(0, len(stressX_lst)):
        if abs(stressX_lst[i]) + abs(stressY_lst[i]) + abs(stressZ_lst[i]) != 0.:
            princ_lst = [ -1.]    
            princ_lst.append(stressX_lst[i] + stressY_lst[i] + stressZ_lst[i])
            princ_lst.append(stressY_lst[i] * stressZ_lst[i] + stressX_lst[i] * stressZ_lst[i] + stressX_lst[i]* stressY_lst[i])
            princ_lst.append(stressX_lst[i] * stressY_lst[i] * stressZ_lst[i])

            answer_lst = CubicEquation(princ_lst).answers

            max_princS = 0.
            for item in answer_lst:
                if item.real > max_princS: max_princS = item.real
                
            min_princS = 0.
            for item in answer_lst:
                if item.real < min_princS: min_princS = item.real

        if abs(stressX_lst[i]) + abs(stressY_lst[i]) + abs(stressZ_lst[i]) == 0.:
            max_princS = 0.
            min_princS = 0. 

        max_princ_stress_lst.append(max_princS)
        min_princ_stress_lst.append(min_princS)

    return(max_princ_stress_lst, min_princ_stress_lst) 
            

def Y_Strain_Longitudinal(t, T, mpd, X, hlp, T_f_ref):
    EPS_Yarn_par = (( (mpd['E_f_long'] * mpd['CTE_f_long'] * (hlp[1] - T_f_ref)  * mpd['V_f']  ) - (  hlp[17] * ( 1 - mpd['V_f']) * hlp[16]   ) )  /
        ( ( mpd['E_f_long'] * mpd['V_f']  ) + (  hlp[17] * (1 - mpd['V_f']) )   ))
 
    return(EPS_Yarn_par)


def Y_Strain_Normal(a, t, T, mpd, X, hlp, hlp_inc):
    EPS_Yarn_norm = 0.25 * hlp[16] * (1 - math.e**(-a / mpd['r_f'])) 
    
    return(EPS_Yarn_norm)


def CalcStrain(l_x, l_y, r_f, f_dico, m_dico, f_keys, m_keys, slx_lst, sly_lst, T, T_ref, CTE_f, EPS_vec_glob, E_r, E_f):
    x_coords_lst = []
    y_coords_lst = []
    
    strain_lst = []
    res_strain_ip_lst = []
    
    princ_strain_lst = []
    princ_stress_lst = []

    for x in slx_lst:
        for y in sly_lst:
            coords = [x, y]

            strain_loc = []

            if coords in m_keys:
                for i in range(0, len(m_keys)):
                    if m_keys[i] == coords:
                        key = i
                        xflen = m_dico[key][0]
                        yflen = m_dico[key][1]

                eps_x_therm = CTE_f * (T- T_ref) * r_f * xflen / l_y
                eps_y_therm = CTE_f * (T- T_ref) * r_f * yflen / l_x

                strain_loc.append( EPS_vec_glob[0] /(  ( (1 - xflen) + eps_x_therm ) + ( (E_r / E_f) *(xflen - eps_x_therm) )  ) )
                strain_loc.append( EPS_vec_glob[1] /(  ( (1 - yflen) + eps_y_therm ) + ( (E_r / E_f) *(yflen - eps_y_therm) ) ) )
                strain_loc.append( EPS_vec_glob[2] )

            if coords not in m_keys:
                strain_loc = [0., 0., 0.]

            strain_lst.append(strain_loc)

            res_strain_ip_lst.append( math.sqrt(strain_loc[0]**2 + strain_loc[1]**2))

            x_coords_lst.append(x)
            y_coords_lst.append(y)


def CHAMIS_E11(mpd, hlp):
    return( mpd['E_f_long'] * mpd['V_f'] + hlp[17] * (1 - mpd['V_f'])  )


def CHAMIS_E22(mpd, hlp):
    return( hlp[17] /  ( 1 - ( math.sqrt(mpd['V_f'])  * ( 1 - (hlp[17] / mpd['E_f_trans']) )   )     )   )
        
        
def CHAMIS_G12(mpd, hlp):
    G_m = hlp[17] / ( 2* ( 1 + hlp[18]))
    G_f = mpd['E_f_long'] / ( 2* ( 1 + mpd['NUE_f_12']))
    return( G_m /  ( 1 - ( math.sqrt(mpd['V_f'])  * ( 1 - (G_m /G_f) )   )     )   )
    
    
def CHAMIS_G23(mpd, hlp):
    G_m = hlp[17] / ( 2* ( 1 + hlp[18]))
    G_f = mpd['E_f_trans'] / ( 2* ( 1 + mpd['NUE_f_23']))
    return( G_m /  ( 1 - ( math.sqrt(mpd['V_f'])  * ( 1 - (G_m /G_f) )   )     )   )


def CHAMIS_NUE12(mpd, hlp):
    return( mpd['V_f'] * mpd['NUE_f_12']  +  (1 - mpd['V_f']) * hlp[18])


def CHAMIS_NUE23(mpd, hlp):
    return( (hlp[60] / ( 2 * hlp[62])) - 1   )