import numpy as np
from numpy.linalg import norm as vecnorm
import math

def VectorRotation(Vector,Normal,HowManyDegrees):

    HowManyRadians = math.radians(HowManyDegrees)
    
    Ux,Uy,Uz = Normal[0], Normal[1], Normal[2]
    Cost= np.cos(HowManyRadians)
    Sint= np.sin(HowManyRadians)

    R =  np.array([[(Cost + Ux**2*(1-Cost)),(Ux*Uy*(1-Cost)-Uz*Sint),(Ux*Uz*(1-Cost)+Uy*Sint)],
                   [(Uy*Ux*(1-Cost)+Uz*Sint),(Cost + Uy**2*(1-Cost)),(Uy*Uz*(1-Cost)-Ux*Sint)],
                   [(Uz*Ux*(1-Cost)-Uy*Sint),(Uz*Uy*(1-Cost)+Ux*Sint),(Cost + Uz**2*(1-Cost))]])

    return np.dot(R,Vector)


def wavinator(E1, E2, E3, nue12, nue13,nue23, G12, G13, G23, thic, rot, wv_rot):

    Dx = np.empty([6,6], dtype=float)

    Dx = np.array( [[0.,0.,0.,0.,0.,0.],
                              [0.,0.,0.,0.,0.,0.],
                              [0.,0.,0.,0.,0.,0.],
                              [0.,0.,0.,0.,0.,0.],
                              [0.,0.,0.,0.,0.,0.],
                              [0.,0.,0.,0.,0.,0.]])

    nue21 = E2 * (nue12 / E1)
    nue31 = E3 * (nue13 / E1)
    nue32 = E3 * (nue23 / E2)

    GAM = 1 / ( 1 - nue12 * nue21 - nue23 * nue32 - nue31 * nue13 - 2 * nue21 * nue32 * nue13)

    Dx[0,0] = E1 * ( 1 - nue23 * nue32 ) * GAM
    Dx[1,1] = E2 * ( 1 - nue13 * nue31 ) * GAM
    Dx[2,2] = E3 * ( 1 - nue12 * nue21 ) * GAM
    
    Dx[3,3] = G12
    Dx[4,4] = G13
    Dx[5,5] = G23
    
    Dx[0,1] = E1 * ( nue21 +  nue31 * nue23 ) * GAM
    Dx[0,2] = E1 * ( nue31 +  nue21 * nue32 ) * GAM
    Dx[1,2] = E2 * ( nue32 +  nue12 * nue31 ) * GAM
    
    Dx[1,0] = Dx[0,1]
    Dx[2,0] = Dx[0,2]
    Dx[2,1] = Dx[1,2]
    
    cos = math.cos(rot*math.pi/180.)
    sin = math.sin(rot*math.pi/180.) 

    T_sig = np.array([  
        [cos**2,     sin**2,     0.,    2*sin*cos,        0.,    0. ],
        [sin**2,     cos**2,     0.,   -2*sin*cos,        0.,    0. ],
        [  0.,         0.,       1.,        0.,           0.,    0. ],
        [-sin*cos,   sin*cos,    0.,    cos**2-sin**2,    0.,    0. ],
        [   0.,          0.,     0.,        0.,           cos,   sin],
        [   0.,          0.,     0.,        0.,           -sin,  cos] 
    ])
    
    T_eps = np.array([
        [cos**2,     sin**2,     0.,      sin*cos,        0.,    0. ],
        [sin**2,     cos**2,     0.,     -sin*cos,        0.,    0. ],
        [  0.,         0.,       1.,        0.,           0.,    0. ],
        [-2*sin*cos,  2*sin*cos,  0.,    cos**2-sin**2,    0.,    0. ],
        [   0.,          0.,     0.,        0.,           cos,   sin],
        [   0.,          0.,     0.,        0.,           -sin,  cos] 
    ])

    T_sig_inv = np.linalg.inv(T_sig)

    T_eps_dot_Sq = np.dot(T_eps, Dx)

    Drot = np.dot(T_eps_dot_Sq, T_sig_inv)

    c= math.cos(wv_rot)
    s = math.sin(wv_rot) 

    T_sig_Z  =   np.array([[ c*c ,  s*s ,  0.0 , 2*c*s , 0.0  ,  0.0 ],
                        [ s*s ,  c*c ,  0.0 , -2*c*s , 0.0  , 0.0 ],
                        [ 0.0 ,  0.0 ,  1.0 , 0.0 , 0.0  ,    0.0 ],
                        [-c*s ,  c*s ,  0.0 , c*c-s*s , 0.0  , 0.0 ],
                        [ 0.0 ,  0.0 ,  0.0 ,   0.0 ,   c  ,    s ],
                        [ 0.0 ,  0.0 ,  0.0 ,   0.0 ,  -s  ,    c ]], float)

    T_eps_Z = np.array([[ c*c ,  s*s ,  0.0 , c*s , 0.0  ,    0.0 ],
                        [ s*s ,  c*c ,  0.0 , -c*s , 0.0  ,   0.0 ],
                        [ 0.0 ,  0.0 ,  1.0 , 0.0 , 0.0  ,    0.0 ],
                        [-2*c*s, 2*c*s, 0.0 ,c*c-s*s, 0.0 ,   0.0 ],
                        [ 0.0 ,  0.0 ,  0.0 ,  0.0,   c  ,   s    ],
                        [ 0.0 ,  0.0 ,  0.0 ,  0.0,  -s  ,    c ]], float)

    T_sig_Z_inv = np.linalg.inv(T_sig_Z)

    T_eps_Z_dot_Sq = np.dot(T_eps_Z, Drot)

    DrotZ = np.dot(T_eps_Z_dot_Sq, T_sig_Z_inv)

    cos = math.cos(-rot*math.pi/180.)
    sin = math.sin(-rot*math.pi/180.) 

    T_sig = np.array([
        [cos**2,     sin**2,     0.,    2*sin*cos,        0.,    0. ],
        [sin**2,     cos**2,     0.,   -2*sin*cos,        0.,    0. ],
        [  0.,         0.,       1.,        0.,           0.,    0. ],
        [-sin*cos,   sin*cos,    0.,    cos**2-sin**2,    0.,    0. ],
        [   0.,          0.,     0.,        0.,           cos,   sin],
        [   0.,          0.,     0.,        0.,           -sin,  cos] 
    ])
    
    T_eps = np.array([
        [cos**2,     sin**2,     0.,      sin*cos,        0.,    0. ],
        [sin**2,     cos**2,     0.,     -sin*cos,        0.,    0. ],
        [  0.,         0.,       1.,        0.,           0.,    0. ],
        [-2*sin*cos,  2*sin*cos,  0.,    cos**2-sin**2,    0.,    0. ],
        [   0.,          0.,     0.,        0.,           cos,   sin],
        [   0.,          0.,     0.,        0.,           -sin,  cos] 
    ])

    T_sig_inv = np.linalg.inv(T_sig)

    T_eps_dot_Sq = np.dot(T_eps, DrotZ)

    DxZ = np.dot(T_eps_dot_Sq, T_sig_inv)

    return(DxZ)


def ShSec_transl(linewise):
    dico = {}
    
    i = 0
    while i <= len(linewise) - 1:
        if linewise[i][0][0] == '*' and linewise[i][0][1] != '*':
            ShS_spl = linewise[i][0].split(', ')
            ShS_key = ShS_spl[1].split('_')[1]

            j = i + 1
            stack_lst = []
            
            while linewise[j][0][0] != '*':
                stack_lst.append(linewise[j][0].split(', '))
                j += 1
            dico.update({ShS_key : stack_lst})
            i = j
        i += 1

    return(dico)


def ORI_transl(linewise):
    dico = {}
    
    i = 0
    while i <= len(linewise) - 1:
        if linewise[i][0][0] == '*' and linewise[i][0][1] != '*':
            ori_spl = linewise[i][0].split(', ')
            ori_key = ori_spl[1].split('=')[1]

            j = i + 1
            ori_lst = []
            
            while linewise[j][0][0] != '*':
                ori_lst.append(linewise[j][0].split(', '))
                orifloat_lst = []
                for ent in ori_lst[0]:
                    orifloat_lst.append(float(ent))
                j += 1
                
            dico.update({ori_key : orifloat_lst})
            i = j
        i += 1

    return(dico)


def prep_defect_list(def_lst):
    
    def_wavin = [] 
    def_misori = []
    def_plymiss = []
    def_thickn = []
    
    for dico in def_lst:
        wav_lst = []
        misori_lst =[]
        mispl_lst = []
        thic_lst =[]
        
        if dico['type'] == 'Ply Waviness':
            def_wavin.append([  dico['cell_id'], 
                            dico['layer_id'], 
                            dico['absolute_local_ply_orientation'], 
                            dico['amplitude'], 
                            dico['length']
                            ])

        if dico['type'] ==  'Ply Misorientation':   
            def_misori.append([dico['cell_id'], 
                                dico['layer_id'], 
                                dico['local_angular_deviation']
                                     ])
        
        if dico['type'] ==  'Missing Ply':   
            def_plymiss.append([dico['cell_id'], 
                                dico['layer_id'], 
                                    ])
        
        if dico['type'] ==  'Varying Thickness':   
            def_thickn.append([dico['cell_id'], 
                                dico['layer_id'], 
                                dico['fvc']/100.
                                     ])    
            
    return(def_wavin, def_misori, def_plymiss, def_thickn)


def SR_vec_rot(ex, ey, ez, x_new, y_new, z_new, resSIG):
    
    from_lst = [ np.array(ex), np.array(ey), np.array(ez) ]
    to_lst = [np.array(x_new), np.array(y_new), np.array(z_new)]
    
    n = np.array( [[0.,0.,0.],
                   [0.,0.,0.],
                   [0.,0.,0.]
                             ])
    
    for i in range(0, 3): 
        for j in range(0, 3): 
            
            n[i, j] = np.vdot(   (to_lst[i] / vecnorm(to_lst[i])), from_lst[j])
            
            rot_resSIG = np.dot(n, resSIG)

    return(rot_resSIG)