from .FKT_LIB_CureKin import *
from .CureKin_Material_Models import *
import numpy as np

def ST_PyCOMP_STApp(dir_dyn, dir_sta, time_lst, temp_lst, mat_param_dico, param_lst, msic):
    MAT_MODEL = 'LTS'

    peak_header = ['tT_ID', 'max_X', 't(max_X)', 'max_dX/dt', 't(max_dX/dt)' , 'max_E', 'app_nue', 
                   'max_SIG_0EPS', 't_max_SIG_0EPS',  'min_SIG_0EPS', 't_min_SIG_0EPS',
                   'max_SIG_plain_stress', 'app_max_EPS_plain_stress', 't_max_SIG_plain_stress', 'min_SIG_plain_stress', 'app_min_EPS_plain_stress', 't_min_SIG_plain_stress', 
                   'max_SIG_plain_strain', 'app_max_EPS_plain_strain', 't_max_SIG_plain_strain', 'min_SIG_plain_strain', 'app_min_EPS_plain_strain', 't_min_SIG_plain_strain']  

    slx_lst, sly_lst, a, l_x, l_y, dx, dy   = GeomHEX(mat_param_dico['V_f'], mat_param_dico['r_f'], mat_param_dico['num_x'], mat_param_dico['num_y'])

    SLX, SLY = np.meshgrid(slx_lst, sly_lst)

    xflen_lst, yflen_lst, f_m_lst = CalcFlen(np.ravel(SLX), np.ravel(SLY), a, l_x, l_y, mat_param_dico['r_f'], dx, dy )
    
    stressX_lst, stressY_lst, stressZ_lst, stress_IP_lst = M_StressXY(xflen_lst, yflen_lst, f_m_lst,  mat_param_dico['E_f_trans'], mat_param_dico['E_r_prelim'], 
                                                                  0.0, -1.0, 0.0, 0.0, 0.0 )

    max_princ_stress_lst, min_princ_stress_lst = M_PrincStress(stressX_lst, stressY_lst, stressZ_lst)
    
    max_princ_stress = 0.
    min_princ_stress = 0.
    
    max_ip_stress = 0.
    min_ip_stress = 1000000000000000.0
    
    for i in range(0, len(max_princ_stress_lst)):
        
        if max_princ_stress_lst[i] > max_princ_stress: 
            max_princ_stress = max_princ_stress_lst[i]
            max_ps_i = i
            
        if min_princ_stress_lst[i] < min_princ_stress: 
            min_princ_stress = min_princ_stress_lst[i]
            min_ps_i = i
            
        if stress_IP_lst[i] > max_ip_stress: 
            max_ip_stress = stress_IP_lst[i]
            max_ip_i = i
            
        if stress_IP_lst[i] < min_ip_stress and stress_IP_lst[i] > 0.: 
            min_ip_stress = stress_IP_lst[i]
            min_ip_i = i
    
    max_princ_stress_ip_vec = [stressX_lst[max_ps_i], stressY_lst[max_ps_i], 0. ]
    min_princ_stress_ip_vec = [stressX_lst[min_ps_i], stressY_lst[min_ps_i], 0. ]
    
    sig_calc_loc_lst = [max_ps_i]
    if min_ps_i not in sig_calc_loc_lst: sig_calc_loc_lst.append(min_ps_i)
    if max_ip_i not in sig_calc_loc_lst: sig_calc_loc_lst.append(max_ip_i)
    if min_ip_i not in sig_calc_loc_lst and min_ip_i > 0.0: sig_calc_loc_lst.append(min_ip_i)
    
    while len(sig_calc_loc_lst) < 4:
        sig_calc_loc_lst.append(0)

    qSLX = []
    qSLY = []
    qxflen_lst = []
    qyflen_lst = []
    qf_m_lst = []
    
    for loc in sig_calc_loc_lst:
        qSLX.append(np.ravel(SLX)[loc])
        qSLY.append(np.ravel(SLY)[loc])
        qxflen_lst.append(xflen_lst[loc])
        qyflen_lst.append(yflen_lst[loc])
        qf_m_lst.append(f_m_lst[loc])

    ot_pref= str(msic.split('.')[0])

    tT_num = 0
    
    ot_info = []
    
    param_lst.sort()
                
    for key in param_lst:
        ot_info.append([key, mat_param_dico[key]])
    ot_info.append(['**\n**'])

    ot_info.append(['tT_ID', 'timefactor _applied', 'time-temperature-curve INPUT', ])

    ot_peak = [peak_header]

    ot_info_inp = []
    
    t_incr = mat_param_dico['tincr']

    for tT_pair in range(0, 1):
        time_sum_lst, temp_sum_lst = time_temp_lst(time_lst, temp_lst, t_incr)

        res_lst = []
        
        RCK_header = ['t', 'T', 'T_g', 'm', 'n', 'k_c1', 'k_c2', 'f_of_X_T', 'l', 'dX_dt', 'X', 'V_r_vol', 
                      'EPS_r_vol', 'EPS_r_vol_incr', 'CTE_r', 'EPS_r_T_incr', 'EPS_r_tot', 'E_r', 'nue_r',
                      'SIG_0EPS_incr', 'SIG_0EPS_tot', 'SIG_plain_stress_incr', 'SIG_plain_stress', 'EPS_plain_stress_incr',
                      'EPS_plain_stress',  'SIG_plain_strain_incr', 'SIG_plain_strain', 'EPS_plain_strain_incr', 'EPS_plain_strain']
                      
        add_lst = ['EPS_Y_par',  'EPS_Y_norm', 'L1_i', 'L1_SX', 'L1_SY', 'L1_SZ', 'L1_S_InPlane', 'L1_S_max_princ',
                   'L1_min_princ', 'L2_i', 'L2_SX', 'L2_SY', 'L2_SZ', 'L2_S_InPlane', 'L2_S_max_princ', 'L2_min_princ']
    
        for i in add_lst:
            RCK_header.append(i) 
            
        add_lst = [ 'L3_i', 'L3_SX', 'L3_SY', 'L3_SZ', 'L3_S_InPlane', 'L3_S_max_princ', 'L3_min_princ',
                   'L4_i', 'L4_SX', 'L4_SY', 'L4_SZ', 'L4_S_InPlane', 'L4_S_max_princ', 'L4_min_princ' ] 
       
        for i in add_lst:
            RCK_header.append(i)
            
        add_lst = [ 'CH_E11', 'CH_E22', 'CH_G12', 'CH_G23', 'CH_NUE12', 'CH_NUE23' ] 
       
        for i in add_lst:
            RCK_header.append(i)

        for i in range(0, len(time_sum_lst)):
            hlp = []
            t = time_sum_lst[i]
            T = temp_sum_lst[i]
            
            hlp.append(t)
            hlp.append(T)
            
            if len(res_lst) == 0:
                X = mat_param_dico['X_0']

                hlp_inc = []
                for incr in range(0, len(RCK_header)):
                    hlp_inc.append(0.)

            if len(res_lst) > 0:
                X = res_lst[-1][10] + res_lst[-1][9] * (t - res_lst[-1][0])

                hlp_inc = res_lst[-1]

            hlp.append(LTS__T_g(t, T, mat_param_dico, X))      #  2
            hlp.append(LTS__m(t, T, mat_param_dico, X))        #  3
            hlp.append(LTS__n(t, T, mat_param_dico, X))        #  4
            hlp.append(LTS__k_c1(t, T, mat_param_dico, X))     #  5
            hlp.append(LTS__k_c2(t, T, mat_param_dico, X))     #  6
            hlp.append(LTS__f_of_X_T(t, T, mat_param_dico, X)) #  7
            hlp.append(LTS__l(t, T, mat_param_dico, X))        #  8
            hlp.append(LTS__dX_dt(t, T, mat_param_dico, X, hlp))    #  9
            hlp.append(X)                                           # 10
            
            ### VOLUMETRIC SHRINK:
            hlp.append(LTS__V_r_vol(t, T, mat_param_dico, X))    #  11
            hlp.append(LTS__EPS_r_vol(t, T, mat_param_dico, X, hlp))    #  12
            hlp.append(LTS__EPS_r_vol_incr(t, T, mat_param_dico, X, hlp, hlp_inc)) #  13
            
            hlp.append(LTS__CTE(t, T, mat_param_dico, X, hlp))        #  14
            
            hlp.append(LTS__EPS_r_T_incr(t, T, mat_param_dico, X, hlp, hlp_inc)) #  15
            
            hlp.append(LTS__EPS_r_tot(t, T, mat_param_dico, X, hlp, hlp_inc))              #16     
            
            hlp.append(LTS__E_r(t, T, mat_param_dico, X, hlp))        #  17
            hlp.append(LTS__nue_r(t, T, mat_param_dico, X, hlp))        #  18
            
            hlp.append(LTS__SIG_0EPS_incr(t, T, mat_param_dico, X, hlp, hlp_inc)) #  19
            hlp.append(LTS__SIG_0EPS(t, T, mat_param_dico, X, hlp, hlp_inc)) #  20
            
            hlp.append(SIG_plain_stress_incr(t, T, mat_param_dico, X, hlp, hlp_inc)) #  21
            hlp.append(SIG_plain_stress(t, T, mat_param_dico, X, hlp, hlp_inc)) #  22
            hlp.append(EPS_plain_stress_incr(t, T, mat_param_dico, X, hlp, hlp_inc)) #  23
            hlp.append(EPS_plain_stress(t, T, mat_param_dico, X, hlp, hlp_inc)) #  24
            
            hlp.append(SIG_plain_strain_incr(t, T, mat_param_dico, X, hlp, hlp_inc)) #  25
            hlp.append(SIG_plain_strain(t, T, mat_param_dico, X, hlp, hlp_inc)) #  26
            hlp.append(EPS_plain_strain_incr(t, T, mat_param_dico, X, hlp, hlp_inc)) #  27
            hlp.append(EPS_plain_strain(t, T, mat_param_dico, X, hlp, hlp_inc)) #  28

            T_f_ref = temp_sum_lst[0]

            hlp.append(Y_Strain_Longitudinal(t, T, mat_param_dico, X, hlp, T_f_ref)) # 29

            hlp.append(Y_Strain_Normal(a, t, T, mat_param_dico, X, hlp, hlp_inc))   # 30
            
            stressX_lst, stressY_lst, stressZ_lst, stress_IP_lst = M_StressXY(qxflen_lst, qyflen_lst, qf_m_lst, mat_param_dico['E_f_trans'], hlp[17], hlp[29], hlp[30], 
                                                                              mat_param_dico['CTE_f_trans'], hlp[1], T_f_ref )

            max_princ_stress_lst, min_princ_stress_lst = M_PrincStress(stressX_lst, stressY_lst, stressZ_lst)

            for i in range(0, len(stressX_lst)):
                hlp.append(sig_calc_loc_lst[i])
                hlp.append(stressX_lst[i])
                hlp.append(stressY_lst[i])
                hlp.append(stressZ_lst[i])
                hlp.append(stress_IP_lst[i])
                hlp.append(max_princ_stress_lst[i])
                hlp.append(min_princ_stress_lst[i])

            '''
            ##################################################################
            ###
            ### YARN HOMOGENIZATION ACC. TO CHAMIS
            ###
            ##################################################################
            '''
            hlp.append(CHAMIS_E11(mat_param_dico, hlp)) #  59
            hlp.append(CHAMIS_E22(mat_param_dico, hlp)) #  60
            hlp.append(CHAMIS_G12(mat_param_dico, hlp)) #  61
            hlp.append(CHAMIS_G23(mat_param_dico, hlp)) #  62
            hlp.append(CHAMIS_NUE12(mat_param_dico, hlp)) #  63
            hlp.append(CHAMIS_NUE23(mat_param_dico, hlp)) #  64

            res_lst.append(hlp)               

    out_path_fin = dir_dyn + ot_pref  + str('.fin')
    of_fin = open(out_path_fin, "w")            

    ot_fin = [RCK_header]

    ot_fin.append(res_lst[-1])

    for line in ot_fin:
        ot_str = ''
        for ent in line:
            ot_str += str(ent) + ','
        ot_str += '\n'

        of_fin.write(ot_str)
    of_fin.flush
    of_fin.close

    fin_dico = {}

    for i in range(0, len(ot_fin[0])):
        fin_dico.update({ot_fin[0][i] : float(ot_fin[1][i])})

    return(fin_dico)            
   

if __name__ == '__main__':    
    ST_PyCOMP_STApp()